/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/

package de.iai.ilcd.configuration;

import java.io.File;
import java.io.IOException;
import java.net.InetAddress;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.UUID;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.googlecode.flyway.core.Flyway;
import com.googlecode.flyway.core.exception.FlywayException;
import com.googlecode.flyway.core.validation.ValidationException;

import de.fzk.iai.ilcd.service.client.impl.vo.NodeInfo;

/**
 * 
 * @author clemens.duepmeier
 */
public enum ConfigurationService {

	INSTANCE;

	private final Logger logger = LoggerFactory.getLogger(ConfigurationService.class);

	// initialize basePath while loading class
	private final String basePath;

	private URI baseURI = null;

	private String contextPath = null;

	private String versionTag = null;

	private Configuration fileConfig;

	private Configuration appConfig;

	private NodeInfo nodeInfo = new NodeInfo();

	private final String defaultPropertiesFile = System.getProperty("catalina.base") + File.separator + "conf"
			+ File.separator + "soda4LCA.properties";

	ConfigurationService() {
		try {
			appConfig = new PropertiesConfiguration("app.properties");
		} catch (ConfigurationException e) {
			throw new RuntimeException("FATAL ERROR: application properties could not be initialized", e);
		}

		// log application version message
		versionTag = appConfig.getString("version.tag");
		logger.info(versionTag);

		// validate/migrate database schema
		migrateDatabaseSchema();

		URL resourceUrl = Thread.currentThread().getContextClassLoader().getResource("log4j.properties");
		String decodedPath = "";
		// now extract path and decode it
		try {
			// note, that URLs getPath() method does not work, because it don't
			// decode encoded Urls, but URI's does this
			decodedPath = resourceUrl.toURI().getPath();
		} catch (URISyntaxException ex) {
			logger.error("Cannot extract base path from resource files", ex);
		}

		// base path it relative to web application root directory
		basePath = decodedPath.replace("/WEB-INF/classes/log4j.properties", "");
		logger.info("base path of web application: {}", basePath);

		// Obtain our environment naming context
		Context initCtx;
		Context envCtx;
		String propertiesFilePath = null;
		try {
			initCtx = new InitialContext();
			envCtx = (Context) initCtx.lookup("java:comp/env");
			propertiesFilePath = (String) envCtx.lookup("soda4LCAProperties");
		} catch (NamingException e1) {
			logger.error(e1.getMessage());
		}

		if (propertiesFilePath == null) {
			logger.info("using default application properties at {}", defaultPropertiesFile);
			propertiesFilePath = defaultPropertiesFile;
		} else {
			logger.info("reading application configuration properties from {}", propertiesFilePath);
		}

		try {
			// OK, now load configuration file
			fileConfig = new PropertiesConfiguration(propertiesFilePath);
		} catch (ConfigurationException ex) {
			logger.error(
					"Cannot find application configuration properties file under {}, either put it there or set soda4LCAProperties environment entry via JNDI.",
					propertiesFilePath, ex);
			throw new RuntimeException("application configuration properties not found", ex);
		}

	}

	public void configureNodeInfo(String ctxPath) {
		if (contextPath == null) {
			contextPath = ctxPath;
			confNodeInfo();
		}
	}

	private void confNodeInfo() {
		String detectedHostName = null;
		try {
			InetAddress inetAddress = InetAddress.getLocalHost();
			detectedHostName = inetAddress.getHostName();
		} catch (UnknownHostException e) {
			logger.error("Could not detect hostname", e);
		}

		String configuredHostName = fileConfig.getString("service.url.hostname");
		int port = fileConfig.getInteger("service.url.port", 80);

		String hostName;

		if (configuredHostName != null)
			hostName = configuredHostName;
		else
			hostName = detectedHostName;

		try {
			URI newUri = new URI("http", null, hostName, (port == 80 ? -1 : port), contextPath, null, null);
			logger.info("application base URI: " + newUri.toString());
			this.baseURI = newUri;
			nodeInfo.setBaseURL(newUri.toString() + "/resource/");
		} catch (URISyntaxException e) {
			throw new RuntimeException("FATAL ERROR: could not determine base URL for service interface", e);
		}

		try {
			nodeInfo.setNodeID(fileConfig.getString("service.node.id"));
		} catch (Exception e) {
			logger.error("Cannot set nodeid from configuration file", e);
		}
		try {
			nodeInfo.setName(fileConfig.getString("service.node.name"));
		} catch (Exception e) {
			logger.error("Cannot set nodename from configuration file", e);
		}

		nodeInfo.setOperator(fileConfig.getString("service.node.operator"));
		nodeInfo.setDescription(fileConfig.getString("service.node.description"));

		nodeInfo.setAdminName(fileConfig.getString("service.admin.name"));
		nodeInfo.setAdminEMail(fileConfig.getString("service.admin.email"));
		nodeInfo.setAdminPhone(fileConfig.getString("service.admin.phone"));
		nodeInfo.setAdminWWW(fileConfig.getString("service.admin.www"));

	}

	private void migrateDatabaseSchema() {
		try {
			Context ctx = new InitialContext();
			DataSource dataSource = (DataSource) ctx.lookup("java:comp/env/jdbc/soda4LCAdbconnection");

			Flyway flyway = new Flyway();
			flyway.setDataSource(dataSource);
			flyway.setBaseDir("sql/migrations");

			logSchemaStatus(flyway);

			try {
				flyway.validate();
			} catch (ValidationException e) {
				logger.error("database schema: could not successfully validate database status, database needs to be initialized");
				throw new RuntimeException("FATAL ERROR: database schema is not properly initialized", e);
			}

			int migrations = flyway.migrate();

			if (migrations > 0) {
				logger.info("database schema: successfully migrated");
				logSchemaStatus(flyway);
			}

		} catch (FlywayException e) {
			logger.error("error migrating database schema", e);
			throw new RuntimeException("FATAL ERROR: database schema is not properly initialized", e);
		} catch (NamingException e) {
			logger.error("error looking up datasource", e);
			throw new RuntimeException("FATAL ERROR: could not lookup datasource", e);
		}

	}

	private void logSchemaStatus(Flyway flyway) {
		if (flyway.status() != null) {
			logger.info("database schema: current version is " + flyway.status().getVersion());
		} else {
			logger.info("database schema: no migration has been applied yet.");
		}
	}

	public String getVersionTag() {
		return versionTag;
	}

	public String getNodeId() {
		return nodeInfo.getNodeID();
	}

	public String getNodeName() {
		return nodeInfo.getName();
	}

	public Configuration getProperties() {
		return fileConfig;
	}

	public NodeInfo getNodeInfo() {
		return nodeInfo;
	}

	public String getBasePath() {
		return basePath;
	}

	public String getContextPath() {
		return contextPath;
	}

	public String getZipFileDirectory() {
		String zipPath = fileConfig.getString("files.location.zipfiles", getBasePath() + "/WEB-INF/var/zips");
		File dir = new File(zipPath);
		try {
			if (!dir.exists()) {
				FileUtils.forceMkdir(dir);
			}
		} catch (IOException e) {
			logger.error("could not create zip files directory at ", zipPath);
		}

		return zipPath;
	}

	public String getDigitalFileBasePath() {
		return fileConfig.getString("files.location.datafiles", getBasePath() + "/WEB-INF/var/files");
	}

	public String getUploadDirectory() {
		return fileConfig.getString("files.location.uploads", getBasePath() + "/WEB-INF/var/uploads");
	}

	public String getUniqueUploadFileName(String prefix, String extension) {
		StringBuilder buffer = new StringBuilder();
		Date date = new Date();
		buffer.append(prefix).append(date.getTime()).append(UUID.randomUUID()).append(".").append(extension);

		return getUploadDirectory() + "/" + buffer.toString();
	}

	public URI getBaseURI() {
		return baseURI;
	}

}
