/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/

package de.iai.ilcd.model.common;

import java.io.Serializable;

import javax.persistence.AssociationOverride;
import javax.persistence.AttributeOverride;
import javax.persistence.Basic;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToOne;
import javax.persistence.MappedSuperclass;
import javax.persistence.OneToOne;
import javax.persistence.Transient;
import javax.persistence.Version;

import de.fzk.iai.ilcd.service.model.IDataSetVO;
import de.iai.ilcd.configuration.ConfigurationService;

/**
 * 
 * @author clemens.duepmeier
 */
@MappedSuperclass
public abstract class DataSet implements Serializable, IDataSetVO {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	protected Long id;

	// we use the field branch for optimistic concurrent locking;
	// i.e. collisions of concurrent updates will look for incremented branch numbers
	// branch will be incremented with every update of the data set
	@Version
	protected int branch = 0;

	// // if it's an remote data set node contains information about the remote network node
	// @Transient
	// private NetworkNode sourceNode=null;
	@Transient
	private String href;

	// we need this, if our data sets will be send to other servers, tools
	@Transient
	private String sourceId = ConfigurationService.INSTANCE.getNodeId();

	@Embedded
	@AttributeOverride(name = "defaultValue", column = @Column(name = "name"))
	@AssociationOverride(name = "lStringMap", joinTable = @JoinTable(name = "dataset_names", joinColumns = @JoinColumn(name = "dataset_id", referencedColumnName = "ID")))
	protected MultiLanguageString name = new MultiLanguageString();

	@Embedded
	protected Uuid uuid = new Uuid();

	@Embedded
	@AttributeOverride(name = "defaultValue", column = @Column(name = "description"))
	@AssociationOverride(name = "lStringMap", joinTable = @JoinTable(name = "dataset_descriptions", joinColumns = @JoinColumn(name = "dataset_id", referencedColumnName = "ID")))
	protected MultiLanguageText description = new MultiLanguageText();

	@ManyToOne(cascade = CascadeType.ALL)
	protected Classification classification = new Classification();

	protected String permanentUri;

	@Embedded
	protected DataSetVersion version;

	@Enumerated(EnumType.STRING)
	protected DataSetState releaseState = DataSetState.UNRELEASED;

	@OneToOne(cascade = CascadeType.ALL, fetch = FetchType.LAZY)
	XmlFile xmlFile;

	/**
	 * Flag to indicate if this is the most recent version of this data set
	 */
	@Basic
	private boolean mostRecentVersion;
	
	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	// public NetworkNode getSourceNode() {
	// return sourceNode;
	// }
	//
	// public void setSourceNode(NetworkNode sourceNode) {
	// this.sourceNode = sourceNode;
	// }

	public int getBranch() {
		return branch;
	}

	public void setBranch(int branch) {
		this.branch = branch;
	}

	/**
	 * Determine if this is the most recent version of this data set
	 * @return <code>true</code> if this is the most recent version of this data set, else <code>false</code>
	 */
	public boolean isMostRecentVersion() {
		return this.mostRecentVersion;
	}
	
	/**
	 * Set the flag to indicate if this data set is the most recent version of the data set
	 * @param mostRecentVersion flag to set
	 */
	public void setMostRecentVersion(boolean mostRecentVersion) {
		this.mostRecentVersion = mostRecentVersion;
	}
	
	@Override
	public Classification getClassification() {
		return classification;
	}

	public void setClassification(Classification classification) {
		this.classification = classification;
	}

	@Override
	public MultiLanguageText getDescription() {
		return description;
	}

	public void setDescription(MultiLanguageText description) {
		this.description = description;
	}

	@Override
	public MultiLanguageString getName() {
		return name;
	}

	@Override
	public String getDefaultName() {
		return name.getDefaultValue();
	}

	public void setName(MultiLanguageString name) {
		this.name = name;
	}

	public Uuid getUuid() {
		return uuid;
	}

	@Override
	public String getUuidAsString() {
		return uuid.getUuid();
	}

	public void setUuid(Uuid uuid) {
		this.uuid = uuid;
	}

	@Override
	public String getDataSetVersion() {
		return this.version != null ? version.getVersionString() : null;
	}

	public DataSetVersion getVersion() {
		return this.version;
	}

	public void setVersion(DataSetVersion version) {
		this.version = version;
	}

	@Override
	public String getPermanentUri() {
		return permanentUri;
	}

	public void setPermanentUri(String permanentUri) {
		this.permanentUri = permanentUri;
	}

	public DataSetState getReleaseState() {
		return releaseState;
	}

	public void setReleaseState(DataSetState releaseState) {
		this.releaseState = releaseState;
	}

	public XmlFile getXmlFile() {
		return xmlFile;
	}

	public void setXmlFile(XmlFile xmlFile) {
		this.xmlFile = xmlFile;
	}

	@Override
	public String getHref() {
		// @TODO - should be generated using the default host name specified in configuration
		// the access part for the dataset type and the UUID value
		return href;
	}

	@Override
	public void setHref(String href) {
		this.href = href;
	}

	@Override
	public String getSourceId() {
		return this.sourceId;
	}

	@Override
	public void setSourceId(String sourceId) {
		this.sourceId = sourceId;
	}

	@Override
	public int hashCode() {
		int hash = 0;
		hash += (id != null ? id.hashCode() : 0);
		return hash;
	}

	@Override
	public boolean equals(Object object) {
		// object is null or no DataSet
		if (!(object instanceof DataSet)) {
			return false;
		}

		// check if same "subclass"
		if (!this.getClass().equals(object.getClass())) {
			return false;
		}

		DataSet other = (DataSet) object;
		// compare id, if set
		if ((this.id == null && other.id != null) || (this.id != null && !this.id.equals(other.id))) {
			return false;
		}

		// compare UUID, if set
		if ((this.uuid == null && other.uuid != null) || (this.uuid != null && !this.uuid.equals(other.uuid))) {
			return false;
		}

		// compare version, if set
		if ((this.version == null && other.version != null)
				|| (this.version != null && !this.version.equals(other.version))) {
			return false;
		}

		// same class, same id, same UUID, same version [same means also: both null]
		return true;
	}

	@Override
	public String toString() {
		return "de.iai.ilcd.model.common.DataSet[id=" + id + "]";
	}

}
