/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.model.dao;

import java.util.HashMap;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.apache.velocity.tools.generic.ValueParser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.fzk.iai.ilcd.service.model.IContactListVO;
import de.fzk.iai.ilcd.service.model.IContactVO;
import de.fzk.iai.ilcd.service.model.ISourceListVO;
import de.iai.ilcd.model.contact.Contact;
import de.iai.ilcd.model.source.Source;
import de.iai.ilcd.persistence.PersistenceUtil;

/**
 * Data access object for {@link Contact}s
 */
public class ContactDao extends DataSetDao<Contact, IContactListVO, IContactVO> {

	private static final Logger logger = LoggerFactory.getLogger(ContactDao.class);

	/**
	 * Map for the possible incoming keys of order by
	 */
	private final static HashMap<String, String> ORDER_BY_KEY_MAP = ContactDao.createOrderByKeyMap();
	
	public ContactDao() {
		super("Contact", Contact.class, IContactListVO.class, IContactVO.class);
	}
	
	/**
	 * Create the map for the possible incoming keys for order by
	 * @return the map for the possible incoming keys for order by
	 */
	private static HashMap<String, String> createOrderByKeyMap() {
		HashMap<String, String> map = new HashMap<String, String>();
		
		// incoming from lazy data table
		map.put("name.defaultValue","c.name.defaultValue");
		map.put("classification.classHierarchyAsString","c1.name ascDesc,c2.name");
		map.put("email","c.email");
		map.put("www","c.www");
		return map;
	}

	@Override
	protected void preCheckAndPersist(Contact dataSet) {
		// Nothing to do for contacts :)
	}

	@Override
	protected Query createQueryObject(ValueParser params, String sortString, boolean ascending, boolean returnCount, boolean mostRecentVersionOnly) {

		EntityManager em = PersistenceUtil.getEntityManager();
		
		boolean joinClassification = !returnCount && "classification.classHierarchyAsString".equals(sortString);
		
		StringBuffer queryString = new StringBuffer();
		if (returnCount) {
			queryString.append("select count(distinct c) from Contact c ");
		} else {
			queryString.append("select distinct c from Contact c ");
		}

		String orderSql = ContactDao.ORDER_BY_KEY_MAP.get( sortString );
		if ( orderSql == null ){
			orderSql = "c.name.defaultValue";
		}
		
		// for order by on classification
		if ( joinClassification ){
			orderSql = orderSql.replaceAll("ascDesc", ascending ? "asc" : "desc");
			queryString.append( this.buildClassificationJoin("c") );
		}
		
		queryString.append("where ");
		
		// for order by on classification
		if ( joinClassification ){
			queryString.append( this.buildClassificationWhere("c") );
			queryString.append(" AND ");
		}		
		
		

		String namePhrase = params.getString("name");
		if (namePhrase == null) {
			namePhrase = "";
		}
		String descriptionPhrase = params.getString("description");
		if (descriptionPhrase == null) {
			descriptionPhrase = "";
		}

		queryString.append("(");

		queryString.append("c.name.defaultValue like :namePhrase");
		if (descriptionPhrase.length() > 2) {
			queryString.append(" or c.description.defaultValue like :descriptionPhrase");
		}
		queryString.append(")");

		// String[] categories = params.getStrings("selectedSectors");
		//
		// if ((categories != null) && (categories.length > 0)) {
		// if (!(categories.length == 1 && categories[0].equals("all"))) {
		// queryString.append(" and ( ");
		// for (int i = 0; i < categories.length; i++) {
		// if (!categories[i].equals("all")) {
		// queryString.append("(dataset.mainCategory='" + categories[i] + "' or dataset.subCategory='" + categories[i] +
		// "')");
		// if (i < categories.length - 1) {
		// queryString.append(" or ");
		// }
		// }
		// }
		// queryString.append(" )");
		// }
		// }

		if (!returnCount){
			queryString.append(" order by "+orderSql+" "+(ascending?"asc":"desc"));
		}
		logger.debug("search query: {}", queryString);

		Query query;

		if (descriptionPhrase.length() > 2) {
			query = em.createQuery(queryString.toString()).setParameter("namePhrase", "%" + namePhrase + "%")
					.setParameter("descriptionPhrase", "%" + descriptionPhrase + "%");
		} else {
			query = em.createQuery(queryString.toString()).setParameter("namePhrase", "%" + namePhrase + "%");
		}

		return query;
	}

	/*
	 * ================================================== == DEPRECATED METHODS / BACKWARDS COMPATIBILITY ==
	 * ==================================================
	 */

	/**
	 * Get contact by UUID
	 * 
	 * @return contact with provided UUID
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getByUuid(String)}.
	 * @see #getByUuid(String)
	 */
	@Deprecated
	public Contact getContact(String uuid) {
		return this.getByUuid(uuid);
	}

	/**
	 * Get the contact with the provided JPA id
	 * 
	 * @param datasetId
	 *            JPA id
	 * @return contact with the provided JPA id
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getByDataSetId(long)}.
	 * @see #getByDataSetId(long)
	 */
	@Deprecated
	public Contact getContact(long datasetId) {
		return this.getByDataSetId(datasetId);
	}

	/**
	 * Get the contact with the provided JPA id
	 * 
	 * @param id
	 *            JPA id as string
	 * @return contact with the provided JPA id
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getByDataSetId(String)}.
	 * @see #getByDataSetId(String)
	 */
	@Deprecated
	public Contact getContactById(String id) {
		return this.getByDataSetId(id);
	}

	/**
	 * Get the contact with the provided JPA id
	 * 
	 * @param id
	 *            JPA id
	 * @return contact with the provided JPA id
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getByDataSetId(long)}.
	 * @see #getByDataSetId(long)
	 */
	@Deprecated
	public Contact getContactById(Long id) {
		return this.getByDataSetId(id);
	}

	/**
	 * Get list of all contacts
	 * 
	 * @return list of all contacts
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getAll()}.
	 * @see #getAll()
	 */
	@Deprecated
	public List<Contact> getContacts() {
		return this.getAll();
	}

	/**
	 * Get contacts by main class
	 * 
	 * @param mainClass
	 *            main class to get contact by
	 * @return matched data sets
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getByClass(String)}.
	 * @see #getByClass(String)
	 */
	@Deprecated
	public List<Contact> getContactsByClass(String mainClass) {
		return this.getByClass(mainClass);
	}

	/**
	 * Get the number of contacts by main class
	 * 
	 * @param mainClass
	 *            main class to get contact by
	 * @return the number of matched data sets
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getByClass(String)}.
	 * @see #getNumberByClass(String)
	 */
	@Deprecated
	public long getNumberOfContacts(String mainClass) {
		return this.getNumberByClass(mainClass);
	}

	/**
	 * Get contacts by main and sub class
	 * 
	 * @param mainClass
	 *            main class to get contact by
	 * @param subClass
	 *            sub class to get contact by
	 * @return matched data sets
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getByClass(String,String)}.
	 * @see #getByClass(String,String)
	 */
	@Deprecated
	public List<Contact> getContactsByClass(String mainClass, String subClass) {
		return this.getByClass(mainClass, subClass);
	}

	/**
	 * Get number of contacts by main and sub class
	 * 
	 * @param mainClass
	 *            main class to get contact by
	 * @param subClass
	 *            sub class to get contact by
	 * @return number of matched data sets
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getByClass(String,String)}.
	 * @see #getNumberByClass(String,String)
	 */
	@Deprecated
	public long getNumberOfContacts(String mainClass, String subClass) {
		return this.getNumberByClass(mainClass, subClass);
	}

}
