/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/

package de.iai.ilcd.model.dao;

import java.io.PrintWriter;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.NoResultException;
import javax.persistence.Query;

import org.apache.velocity.tools.generic.ValueParser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.fzk.iai.ilcd.service.model.IDataSetListVO;
import de.fzk.iai.ilcd.service.model.IDataSetVO;
import de.fzk.iai.ilcd.service.model.ILCIAMethodListVO;
import de.iai.ilcd.model.common.DataSet;
import de.iai.ilcd.model.common.DataSetType;
import de.iai.ilcd.model.common.DataSetVersion;
import de.iai.ilcd.model.lciamethod.LCIAMethod;
import de.iai.ilcd.persistence.PersistenceUtil;

/**
 * Common implementation for data set DAO objects
 * 
 * @param <T>
 *            Element type that is accessed by this DAO
 * @param <L>
 *            ListVO interface type for the access
 */
public abstract class DataSetDao<T extends DataSet, L extends IDataSetListVO, D extends IDataSetVO> {

	private static final Logger LOGGER = LoggerFactory.getLogger(DataSetDao.class);

	/**
	 * The name in JPA of class
	 */
	private final String jpaName;

	/**
	 * The class accessed by this DAO
	 */
	private final Class<T> accessedClass;

	/**
	 * The class accessed by this DAO
	 */
	private final Class<L> listVOClass;

	/**
	 * The class accessed by this DAO
	 */
	private final Class<D> voClass;

	/**
	 * The data set type
	 */
	private final DataSetType dataSetType;

	/**
	 * Create a data set DAO
	 * 
	 * @param jpaName
	 *            the name in JPA of class
	 * @param accessedClass
	 *            the class accessed by this DAO
	 * @param listVOClass
	 *            the interface based on {@link ILCIAMethodListVO} for the accessed class
	 */
	public DataSetDao(String jpaName, Class<T> accessedClass, Class<L> listVOClass, Class<D> voClass) {
		this.jpaName = jpaName;
		this.accessedClass = accessedClass;
		this.listVOClass = listVOClass;
		this.voClass = voClass;
		this.dataSetType = DataSetType.fromClass(accessedClass);
	}

	/**
	 * Get the name in JPA of class
	 * 
	 * @return name in JPA of class
	 */
	protected String getJpaName() {
		return this.jpaName;
	}

	/**
	 * Get the class accessed by this DAO
	 * 
	 * @return class accessed by this DAO
	 */
	protected Class<T> getAccessedClass() {
		return this.accessedClass;
	}

	/**
	 * Get all data sets in the persistence unit of the represented type T
	 * 
	 * @return all data sets in the persistence unit of the represented type T
	 */
	@SuppressWarnings("unchecked")
	public List<T> getAll() {
		EntityManager em = PersistenceUtil.getEntityManager();
		return (List<T>) em.createQuery("select a from " + this.jpaName + " a").getResultList();
	}

	/**
	 * Get the count of elements in the database for type T
	 * 
	 * @return count of elements in the database for type T
	 */
	public Long getAllCount() {
		EntityManager em = PersistenceUtil.getEntityManager();
		return (Long) em.createQuery("select count(a) from " + this.jpaName + " a").getSingleResult();
	}

	/**
	 * Get <code>pageSize</code> data sets starting on <code>startIndex</code>
	 * 
	 * @param startIndex
	 * @param pageSize
	 * @return list of matching elements
	 */
	@SuppressWarnings("unchecked")
	public List<T> getDataSets(int startIndex, int pageSize) {
		EntityManager em = PersistenceUtil.getEntityManager();
		return (List<T>) em.createQuery("select a from " + this.jpaName + " a").setFirstResult(startIndex)
				.setMaxResults(pageSize).getResultList();
	}

	/**
	 * Get data set by UUID string and version
	 * 
	 * @param uuid
	 *            the UUID string
	 * @param version
	 *            the version of the method
	 * @return data set for the specified UUID string/version, or <code>null</code> if none found
	 */
	@SuppressWarnings("unchecked")
	public T getByUuidAndVersion(String uuid, DataSetVersion version) {
		try {
			EntityManager em = PersistenceUtil.getEntityManager();
			Query q = em
					.createQuery("select a from "
							+ this.jpaName
							+ " a where a.uuid.uuid=:uuid and a.version.majorVersion=:major and a.version.minorVersion=:minor and a.version.subMinorVersion=:subMinor");
			q.setParameter("uuid", uuid);
			q.setParameter("major", version.getMajorVersion());
			q.setParameter("minor", version.getMinorVersion());
			q.setParameter("subMinor", version.getSubMinorVersion());
			return (T) q.getSingleResult();
		} catch (NoResultException e) {
			// none found, so let's return null
			return null;
		}
	}

	/**
	 * Get a the most recent data set of type T by UUID string
	 * 
	 * @param uuid
	 *            the UUID string
	 * @return data set of type T for the specified UUID string, or <code>null</code> if none found
	 */
	@SuppressWarnings("unchecked")
	public T getByUuid(String uuid) {
		try {
			EntityManager em = PersistenceUtil.getEntityManager();
			Query q = em
					.createQuery("select a from "
							+ this.jpaName
							+ " a where a.uuid.uuid=:uuid order by a.version.majorVersion desc, a.version.minorVersion desc, a.version.subMinorVersion desc");
			q.setParameter("uuid", uuid);
			List<T> results = q.getResultList();
			if (results.size() > 0)
				return results.get(0);
			else
				return null;
		} catch (NoResultException e) {
			// none found, so let's return null
			return null;
		}
	}

	/**
	 * Get all other versions of this data set
	 * 
	 * @return version instances of all version numbers that are available for the uuid of this data set except the
	 *         current one
	 */
	@SuppressWarnings("unchecked")
	public List<T> getOtherVersions(T current) {
		if (current == null) {
			return null;
		}

		final String uuid = current.getUuidAsString();
		final DataSetVersion v = current.getVersion();

		if (uuid == null || v == null) {
			return null;
		}

		try {
			EntityManager em = PersistenceUtil.getEntityManager();
			Query q = em
					.createQuery("select a from "
							+ this.jpaName
							+ " a WHERE a.uuid.uuid=:uuid AND (a.version.majorVersion<>:major OR a.version.minorVersion<>:minor OR a.version.subMinorVersion<>:subMinor) ORDER BY a.version.majorVersion desc, a.version.minorVersion desc, a.version.subMinorVersion desc");

			q.setParameter("uuid", uuid);
			q.setParameter("major", v.getMajorVersion());
			q.setParameter("minor", v.getMinorVersion());
			q.setParameter("subMinor", v.getSubMinorVersion());

			return (List<T>) q.getResultList();
		} catch (NoResultException e) {
			return null;
		}
	}

	/**
	 * Get the data set of type T by JPA id
	 * 
	 * @param id
	 *            id of the data set
	 * @return data set of type T by JPA id
	 */
	public T getByDataSetId(long id) {
		EntityManager em = PersistenceUtil.getEntityManager();
		return (T) em.find(this.getAccessedClass(), new Long(id));
	}

	/**
	 * Get the data set of type T by JPA id
	 * 
	 * @param id
	 *            id of the data set as string
	 * @return data set of type T by JPA id
	 */
	public T getByDataSetId(String id) {
		try {
			return this.getByDataSetId(Long.parseLong(id));
		} catch (NumberFormatException e) {
			return null;
		}
	}

	/**
	 * Get the top classes for the data set type T
	 * 
	 * @return top classes for the data set type T
	 */
	public List<String> getTopClasses() {
		ClassificationDao classDao = new ClassificationDao();
		return classDao.getTopClasses(this.dataSetType);
	}

	/**
	 * Get the sub classes for provided class name and level
	 * 
	 * @param className
	 *            class name
	 * @param level
	 *            level
	 * @return sub classes for provided class name and level
	 */
	@SuppressWarnings("unchecked")
	public List<String> getSubClasses(String className, String level) {
		EntityManager em = PersistenceUtil.getEntityManager();

		Query q = em
				.createQuery("select distinct cl.name from "
						+ this.jpaName
						+ " a join a.classification.classes cl join a.classification.classes cl2 where cl.level=:level and cl2.name=:className order by cl.name");
		q.setParameter("level", Integer.parseInt(level));
		q.setParameter("className", className);

		return (List<String>) q.getResultList();
	}

	/**
	 * Get data sets of type T by main class
	 * 
	 * @param mainClass
	 *            the main class to get data sets by
	 * @return data sets of type T by main class
	 */
	@SuppressWarnings("unchecked")
	public List<T> getByClass(String mainClass) {
		return (List<T>) this.getByClassQuery(mainClass, false).getResultList();
	}

	/**
	 * Get number of data sets of type T by main class
	 * 
	 * @param mainClass
	 *            the main class to get data sets by
	 * @return number of data sets of type T by main class
	 */
	public long getNumberByClass(String mainClass) {
		return (Long) this.getByClassQuery(mainClass, true).getSingleResult();
	}

	/**
	 * Create query (with set parameters) for {@link #getByClass(String)} and {@link #getNumberByClass(String)}
	 * 
	 * @param mainClass
	 *            the main class to get data sets by
	 * @param count
	 *            do count query or not
	 * @return query (with set parameters) for {@link #getByClass(String)} and {@link #getNumberByClass(String)}
	 */
	private Query getByClassQuery(String mainClass, boolean count) {
		EntityManager em = PersistenceUtil.getEntityManager();
		return em.createQuery(
				"select " + (count ? "count(a)" : "a") + " from " + this.jpaName
						+ " a join a.classification.classes cl where cl.level=0 and cl.name=:className").setParameter(
				"className", mainClass);
	}

	/**
	 * Get data sets of type T by main and sub class
	 * 
	 * @param mainClass
	 *            the main class to get data sets by
	 * @param subClass
	 *            the sub class to get data sets by
	 * @return data sets of type T by main and sub class
	 */
	@SuppressWarnings("unchecked")
	public List<T> getByClass(String mainClass, String subClass) {
		return (List<T>) this.getByClassQuery(mainClass, subClass, false).getResultList();
	}

	/**
	 * Get data sets of type T by main and sub class
	 * 
	 * @param mainClass
	 *            the main class to get data sets by
	 * @param subClass
	 *            the sub class to get data sets by
	 * @return data sets of type T by main and sub class
	 */
	public long getNumberByClass(String mainClass, String subClass) {
		return (Long) this.getByClassQuery(mainClass, subClass, true).getSingleResult();
	}

	/**
	 * Create query (with set parameters) for {@link #getByClass(String,String)} and
	 * {@link #getNumberByClass(String,String)}
	 * 
	 * @param mainClass
	 *            the main class to get data sets by
	 * @param subClass
	 *            the sub class to get data sets by
	 * @param count
	 *            do count query or not
	 * @return query (with set parameters) for {@link #getByClass(String,String)} and
	 *         {@link #getNumberByClass(String,String)}
	 */
	private Query getByClassQuery(String mainClass, String subClass, boolean count) {
		EntityManager em = PersistenceUtil.getEntityManager();
		return em
				.createQuery(
						"select "
								+ (count ? "count(a)" : "a")
								+ " from "
								+ this.jpaName
								+ " a join a.classification.classes cl join a.classification.classes cl2 where cl.level=0 and cl.name=:mainClass and cl2.level=1 and cl2.name=:subClass")
				.setParameter("mainClass", mainClass).setParameter("subClass", subClass);
	}

	/**
	 * Search for data sets of type T
	 * 
	 * @param params
	 *            parameter
	 * @param mostRecentVersionOnly
	 *            flag to indicate if only the most recent version of a data set shall be returned if multiple versions
	 *            exist
	 * @return found data sets of type T
	 */
	public List<L> search(ValueParser params, boolean mostRecentVersionOnly) {
		return this.search(this.listVOClass, params, mostRecentVersionOnly);
	}

	/**
	 * Search for data sets of type T
	 * 
	 * @param params
	 *            parameter
	 * @param startIndex
	 *            start index for query
	 * @param pageSize
	 *            count of result items
	 * @param mostRecentVersionOnly
	 *            flag to indicate if only the most recent version of a data set shall be returned if multiple versions
	 *            exist
	 * @return found data sets of type T
	 */
	public List<L> search(ValueParser params, int startIndex, int pageSize, boolean mostRecentVersionOnly) {
		return this.search(this.listVOClass, params, startIndex, pageSize, null, mostRecentVersionOnly);
	}

	/**
	 * Invoked at beginning of {@link #checkAndPersist(DataSet, PersistType, PrintWriter)} to manipulate data prior to
	 * persisting
	 * 
	 * @param dataSet
	 *            data set to manipulate
	 * @see #checkAndPersist(DataSet, PersistType, PrintWriter)
	 */
	protected abstract void preCheckAndPersist(T dataSet);

	/**
	 * Generic persist method for data set objects.
	 * 
	 * @param dataSet
	 *            data set to persist
	 * @param pType
	 *            which type of persistence operation, new, update (i.e. overwrite existing data set), ...
	 * @param out
	 *            PrintWriter to log error messages which can be presented to the end user
	 * @return true if persist is successful, false otherwise
	 * @see #preCheckAndPersist(DataSet)
	 */
	public boolean checkAndPersist(T dataSet, PersistType pType, PrintWriter out) {
		// TODO: check if version shall be excluded for some types

		// perform pre-persist actions
		this.preCheckAndPersist(dataSet);

		EntityManager em = PersistenceUtil.getEntityManager();

		// locate existing method by UUID
		T existingDataSet = this.getByUuidAndVersion(dataSet.getUuidAsString(), dataSet.getVersion());

		// if existing found ...
		if (existingDataSet != null) {
			// ... and mode is set to only new ...
			if (PersistType.ONLYNEW.equals(pType)) {
				if (out != null) {
					out.println("Warning: " + this.accessedClass.getSimpleName() + " data set with uuid "
							+ dataSet.getUuidAsString() + " and version " + dataSet.getVersion().getVersionString()
							+ " already exists in database; will ignore this data set.");
				}
				// ... just ignore it
				return false;
			}
		}

		EntityTransaction eTransaction = em.getTransaction();
		// now the DB interaction
		try {
			// begin the transaction
			eTransaction.begin();

			// delete existing for merge operation
			if (existingDataSet != null && PersistType.MERGE.equals(pType)) {
				if (out != null) {
					out.println("Notice: " + this.accessedClass.getSimpleName() + " method data set with uuid "
							+ existingDataSet.getUuidAsString() + " and version "
							+ existingDataSet.getVersion().getVersionString()
							+ " already exists in database; will replace it with this data set");
				}
				em.remove(existingDataSet);
			}
			// persist the new method
			em.persist(dataSet);
			// set the most recent version flags correctly
			this.setMostRecentVersionFlags(dataSet.getUuidAsString());
			// commit the transaction
			eTransaction.commit();
			// and return with success :)
			return true;
		} catch (Exception e) {
			DataSetDao.LOGGER.error("Cannot persist " + this.accessedClass.getSimpleName() + " data set with uuid {}",
					dataSet.getUuidAsString());
			DataSetDao.LOGGER.error("Exception is: ", e);
			if (out != null) {
				out.println("Error: " + this.accessedClass.getSimpleName() + " data set with uuid "
						+ dataSet.getUuidAsString() + " and version " + dataSet.getVersion().getVersionString()
						+ " could not be saved into database; unknown database error; Exception message: "
						+ e.getMessage());
			}

			// roll back the operation if possible
			if (eTransaction != null && eTransaction.isActive()) {
				eTransaction.rollback();
			}
		}

		return false;
	}

	/**
	 * Set the {@link DataSet#setMostRecentVersion(boolean) most recent version flags} for all data sets with the
	 * specified uuid. Please note that this method <strong>does not open a separate transaction</strong>.
	 * 
	 * @param uuid
	 *            uuid of the data sets
	 */
	@SuppressWarnings("unchecked")
	private void setMostRecentVersionFlags(String uuid) {
		EntityManager em = PersistenceUtil.getEntityManager();

		try {
			Query q = em
					.createQuery("SELECT a FROM "
							+ this.jpaName
							+ " a WHERE a.uuid.uuid=:uuid ORDER BY a.version.majorVersion desc, a.version.minorVersion desc, a.version.subMinorVersion desc");
			q.setParameter("uuid", uuid);
			List<T> res = (List<T>) q.getResultList();
			if (!res.isEmpty()) {
				// get first element and mark it as most recent version (correct order is taken care of in query!)
				T tmp = res.get(0);
				tmp.setMostRecentVersion(true);
				em.merge(tmp);

				// set "false" for all other elements if required
				final int size = res.size();
				if (size > 1) {
					for (int i = 1; i < size; i++) {
						tmp = res.get(i);
						if (tmp.isMostRecentVersion()) {
							tmp.setMostRecentVersion(false);
							em.merge(tmp);
						}
					}
				}
			}
		} catch (NoResultException e) {
			DataSetDao.LOGGER.warn("Most recent version flag was called for non-existent UUID: " + uuid);
		} catch (Exception e) {
			DataSetDao.LOGGER.error("Could not set most recent version flag for UUID: " + uuid);
		}
	}

	/**
	 * This method (local serach) works as search method for datasets only on local database entities, i.e. Process,
	 * Flow, ... To use a search method which also works distributed use the corresponding search method instead
	 * 
	 * @param <T>
	 *            Class name of "type" of objects to return, i.e. Process, Flow, ...
	 * @param params
	 *            search parameters as key-value pairs
	 * @return list of datasets of type T
	 */
	public List<T> lsearch(ValueParser params) {
		return this.lsearch(params, 0, 0, null, true);
	}

	/**
	 * This method (local lsearch) works as lsearch method for datasets only on local database entities, i.e. Process,
	 * Flow, ... To use a lsearch method which also works distributed use the corresponding lsearch method
	 * 
	 * @param <T>
	 *            Class name of "type" of objects to return, i.e. Process, Flow, ...
	 * @param params
	 *            lsearch parameters as key-value pairs
	 * @param startPosition
	 *            start index within the whole list of lsearch results
	 * @param pageSize
	 *            pages size, i.e. maximum count of results to return
	 * @param sortCriterium
	 *            name of the objects field which should be used for sorting
	 * @param mostRecentVersionOnly
	 *            flag to indicate if only the most recent version of a data set shall be returned if multiple versions
	 *            exist
	 * @return list of datasets of type T matching the lsearch criteria
	 */
	public List<T> lsearch(ValueParser params, int startPosition, int pageSize, String sortCriterium,
			boolean mostRecentVersionOnly) {
		return this.lsearch(params, startPosition, pageSize, sortCriterium, true, mostRecentVersionOnly);
	}

	/**
	 * This method (local lsearch) works as lsearch method for datasets only on local database entities, i.e. Process,
	 * Flow, ... To use a lsearch method which also works distributed use the corresponding lsearch method
	 * 
	 * @param <T>
	 *            Class name of "type" of objects to return, i.e. Process, Flow, ...
	 * @param params
	 *            lsearch parameters as key-value pairs
	 * @param startPosition
	 *            start index within the whole list of lsearch results
	 * @param pageSize
	 *            pages size, i.e. maximum count of results to return
	 * @param sortCriterium
	 *            name of the objects field which should be used for sorting
	 * @param ascending
	 *            define ordering of result
	 * @param mostRecentVersionOnly
	 *            flag to indicate if only the most recent version of a data set shall be returned if multiple versions
	 *            exist
	 * @return list of datasets of type T matching the lsearch criteria
	 */
	@SuppressWarnings("unchecked")
	public List<T> lsearch(ValueParser params, int startPosition, int pageSize, String sortCriterium,
			boolean ascending, boolean mostRecentVersionOnly) {
		List<T> dataSets = null;

		Query query = this.createQueryObject(params, sortCriterium, ascending, false, mostRecentVersionOnly);

		if (startPosition > 0) {
			query.setFirstResult(startPosition);
		}
		if (pageSize > 0) {
			query.setMaxResults(pageSize);
		}

		dataSets = (List<T>) query.getResultList();

		return dataSets;
	}

	/**
	 * 
	 * This search method can also be used for distributed search because it uses the common interface types of the
	 * ServiceAPI For a version without the dataSetClassType parameter look in the subclasses
	 * 
	 * @param <E>
	 *            interface name of "type" of objects to return, i.e. IProcessListVO, IFlowListVo, ...
	 * @param dataSetClassType
	 *            Class object of T, i.e. IProcessListVO.class
	 * @param params
	 *            search parameter
	 * @param mostRecentVersionOnly
	 *            flag to indicate if only the most recent version of a data set shall be returned if multiple versions
	 *            exist
	 * @return List of objects of the give interface type E
	 * 
	 */
	public <E extends IDataSetListVO> List<E> search(Class<E> dataSetClassType, ValueParser params,
			boolean mostRecentVersionOnly) {
		return this.search(dataSetClassType, params, 0, 0, null, mostRecentVersionOnly);
	}

	/**
	 * This search method can also be used for distributed search because it uses the common interface types of the
	 * ServiceAPI
	 * 
	 * @param <E>
	 *            interface name of "type" of objects to return, i.e. IProcessListVO, IFlowListVo, ...
	 * @param dataSetClassType
	 *            Class object of T, i.e. IProcessListVO.class
	 * @param params
	 *            search parameter as ParameterTool object
	 * @param startPosition
	 *            start index for first search result
	 * @param pageSize
	 *            size of one page of search results
	 * @param sortCriterium
	 *            field name of object which is used for ordering
	 * @param mostRecentVersionOnly
	 *            flag to indicate if only the most recent version of a data set shall be returned if multiple versions
	 *            exist
	 * @return list of objects of the given interface E
	 */
	public <E extends IDataSetListVO> List<E> search(Class<E> dataSetClassType, ValueParser params, int startPosition,
			int pageSize, String sortCriterium, boolean mostRecentVersionOnly) {
		return this.search(dataSetClassType, params, startPosition, pageSize, sortCriterium, true,
				mostRecentVersionOnly);
	}

	/**
	 * This search method can also be used for distributed search because it uses the common interface types of the
	 * ServiceAPI
	 * 
	 * @param <E>
	 *            interface name of "type" of objects to return, i.e. IProcessListVO, IFlowListVo, ...
	 * @param dataSetClassType
	 *            Class object of T, i.e. IProcessListVO.class
	 * @param params
	 *            search parameter as ParameterTool object
	 * @param startPosition
	 *            start index for first search result
	 * @param pageSize
	 *            size of one page of search results
	 * @param sortCriterium
	 *            field name of object which is used for ordering
	 * @param ascending
	 *            define the ordering of the result
	 * @param mostRecentVersionOnly
	 *            flag to indicate if only the most recent version of a data set shall be returned if multiple versions
	 *            exist
	 * @return list of objects of the given interface E
	 */
	@SuppressWarnings("unchecked")
	public <E extends IDataSetListVO> List<E> search(Class<E> dataSetClassType, ValueParser params, int startPosition,
			int pageSize, String sortCriterium, boolean ascending, boolean mostRecentVersionOnly) {

		LOGGER.trace("searching for {}", dataSetClassType.getName());

		List<E> dataSets = null;

		Query query = this.createQueryObject(params, sortCriterium, ascending, false, mostRecentVersionOnly);

		if (startPosition > 0) {
			query.setFirstResult(startPosition);
		}
		if (pageSize > 0) {
			query.setMaxResults(pageSize);
		}

		dataSets = (List<E>) query.getResultList();

		if (LOGGER.isTraceEnabled()) {
			for (IDataSetListVO ds : dataSets) {
				LOGGER.trace("name: {}, defaultname: {}", ds.getName().getValue(), ds.getDefaultName());
			}
		}

		// OK, search also in other systems
		if (params.exists("distributed") && params.getBoolean("distributed")) {
			LOGGER.info("initiating distributed search");
			ForeignDataSetsHelper foreignHelper = new ForeignDataSetsHelper();
			List<E> foreignProcesses = foreignHelper.foreignSearch(dataSetClassType, params);
			if (foreignProcesses != null)
				LOGGER.debug("returning {} results from foreign nodes", foreignProcesses.size());
			dataSets.addAll(foreignProcesses);
		}

		return dataSets;
	}

	/**
	 * Get result count for lsearch call
	 * 
	 * @param params
	 *            lsearch parameters
	 * @param mostRecentVersionOnly
	 *            flag to indicate if only the most recent version of a data set shall be returned if multiple versions
	 *            exist
	 * @return returns the count of results when a lsearch is issued with this lsearch parameters
	 */
	public long searchResultCount(ValueParser params, boolean mostRecentVersionOnly) {
		Query query = createQueryObject(params, null, true, true, mostRecentVersionOnly);

		Long resultCount = (Long) query.getSingleResult();

		return resultCount.longValue();
	}

	/**
	 * 
	 * This method must be implemented by each dao class extending this class. It will return the specific Query object
	 * to use for the given subclass of objects, i.e. Process, Flow, ... This method will then be used by the other
	 * generic lsearch functions to issue the special query.
	 * 
	 * @param params
	 *            lsearch parameter
	 * @param sortCriterium
	 *            field of result object which will be used for ordering of lsearch results
	 * @param returnCount
	 *            if true return count of lsearch result instead of lsearch results
	 * @param ascending
	 *            <code>true</code> for ascending ordering, <code>false</code> for descending
	 * @param mostRecentVersionOnly
	 *            flag to indicate if only the most recent version of a data set shall be returned if multiple versions
	 *            exist
	 * @return JPA Query objects for doing the lsearch
	 * 
	 */
	abstract protected Query createQueryObject(ValueParser params, String sortCriterium, boolean ascending,
			boolean returnCount, boolean mostRecentVersionOnly);

	/**
	 * Remove a given data set from the database
	 * 
	 * @param dataSet
	 *            data set to be removed
	 */
	public void remove(DataSet dataSet) throws DeleteDataSetException {
		if (dataSet == null || dataSet.getId() == null) {
			return;
		}
		EntityManager em = PersistenceUtil.getEntityManager();
		EntityTransaction tx = em.getTransaction();
		try {
			tx.begin();
			DataSet toRemove = em.find(dataSet.getClass(), dataSet.getId());
			if (toRemove != null) {
				final String uuid = toRemove.getUuidAsString();
				em.remove(toRemove);
				this.setMostRecentVersionFlags(uuid);
			}
			tx.commit();
		} catch (Exception e) {
			DataSetDao.LOGGER.error("Cannot remove data set {} from database", dataSet.getName().getDefaultValue(), e);
			if (tx.isActive()) {
				tx.rollback();
				throw new DeleteDataSetException("Cannot remove data set " + dataSet.getName().getDefaultValue());
			}
		}
	}

	public void merge(DataSet dataSet) throws MergeException {
		if ((dataSet == null) || (dataSet.getId() == null)) {
			return;
		}
		EntityManager em = PersistenceUtil.getEntityManager();
		EntityTransaction tx = em.getTransaction();
		try {
			tx.begin();
			em.merge(dataSet);
			tx.commit();
		} catch (Exception e) {
			if (tx.isActive()) {
				tx.rollback();
			}
			throw new MergeException("Cannot merge changes of data set " + dataSet.getName().getDefaultValue()
					+ " into database");
		}
	}

	/**
	 * Get a foreign data set
	 * 
	 * @param nodeShortName
	 *            short name / id of remote node
	 * @param uuid
	 *            uuid of data set
	 * @return loaded data set
	 */
	public D getForeignDataSet(String nodeShortName, String uuid) {
		ForeignDataSetsHelper helper = new ForeignDataSetsHelper();
		return helper.getForeignDataSet(this.voClass, nodeShortName, uuid);
	}

	/**
	 * <p>
	 * Build up string for classification where clause (with default values for classification field name and alias
	 * prefix), example:
	 * </p>
	 * <p>
	 * <code>( p.classification IS NULL OR (c1.level=0 AND c2.level=0) OR (c1.level=0 AND c2.level=1) )</code>
	 * </p>
	 * <ul>
	 * <li>default classification field name = <code>classification</code></li>
	 * <li>default alias prefix = <code>c</code></li>
	 * </ul>
	 * 
	 * @param entityPrefix
	 *            prefix of the entity, in example it is <code>p</code>
	 * @return created where clause part with space at the end
	 * @see #buildClassificationWhere(String, String, String)
	 */
	protected String buildClassificationWhere(String entityPrefix) {
		return this.buildClassificationWhere(entityPrefix, "classification", "c");
	}

	/**
	 * <p>
	 * Build up string for classification where clause, example:
	 * </p>
	 * <p>
	 * <code>( p.classification IS NULL OR (c1.level=0 AND c2.level=0) OR (c1.level=0 AND c2.level=1) )</code>
	 * </p>
	 * 
	 * @param entityPrefix
	 *            prefix of the entity, in example it is <code>p</code>
	 * @param classificFieldName
	 *            name of the classification field in the entity, in example it is <code>classification</code>
	 * @param aliasPrefix
	 *            alias prefix for the left joined references, in example it is <code>c</code>
	 * @return created where clause part with space at the end
	 */
	protected String buildClassificationWhere(String entityPrefix, String classificFieldName, String aliasPrefix) {
		return "( " + entityPrefix + "." + classificFieldName + " IS NULL OR (" + aliasPrefix + "1.level=0 AND "
				+ aliasPrefix + "2.level=0) OR (" + aliasPrefix + "1.level=0 AND " + aliasPrefix + "2.level=1) ) ";
	}

	/**
	 * <p>
	 * Build up join string for classification containing query (with default values for classification field name and
	 * alias prefix), example:
	 * </p>
	 * <p>
	 * <code>LEFT JOIN p.classification.classes c1 LEFT JOIN p.classification.classes c2</code>
	 * </p>
	 * <ul>
	 * <li>default classification field name = <code>classification</code></li>
	 * <li>default alias prefix = <code>c</code></li>
	 * </ul>
	 * 
	 * @param entityPrefix
	 *            prefix of the entity, in example it is <code>p</code>
	 * @return created join select clause part with space at the end
	 * @see #buildClassificationJoin(String, String, String)
	 */
	protected String buildClassificationJoin(String entityPrefix) {
		return this.buildClassificationJoin(entityPrefix, "classification", "c");
	}

	/**
	 * <p>
	 * Build up join string for classification containing query, example:
	 * </p>
	 * <p>
	 * <code>LEFT JOIN p.classification.classes c1 LEFT JOIN p.classification.classes c2</code>
	 * </p>
	 * 
	 * @param entityPrefix
	 *            prefix of the entity, in example it is <code>p</code>
	 * @param classificFieldName
	 *            name of the classification field in the entity, in example it is <code>classification</code>
	 * @param aliasPrefix
	 *            alias prefix for the left joined references, in example it is <code>c</code>
	 * @return created join select clause part with space at the end
	 */
	protected String buildClassificationJoin(String entityPrefix, String classificFieldName, String aliasPrefix) {
		return "LEFT JOIN " + entityPrefix + "." + classificFieldName + ".classes " + aliasPrefix + "1 LEFT JOIN "
				+ entityPrefix + "." + classificFieldName + ".classes " + aliasPrefix + "2 ";
	}
}
