/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.webgui.controller.admin;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.context.FacesContext;
import javax.faces.event.AbortProcessingException;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.primefaces.event.FileUploadEvent;
import org.primefaces.model.UploadedFile;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.fzk.iai.ilcd.api.binding.helper.DatasetDAO;
import de.fzk.iai.ilcd.api.dataset.ILCDTypes;
import de.iai.ilcd.configuration.ConfigurationService;
import de.iai.ilcd.xml.read.DataSetImporter;

/**
 * 
 * @author clemens.duepmeier
 */
@ManagedBean
@SessionScoped
public class ImportHandler implements Serializable {
	private static final long serialVersionUID = 1L;

	public static Logger logger = LoggerFactory.getLogger(ImportHandler.class);

	private String handlerId;

	private List<UploadedFileInformation> uploadedFiles = new ArrayList<UploadedFileInformation>();

	public ImportHandler() {
		handlerId = UUID.randomUUID().toString();
	}

	public void handleFileUpload(FileUploadEvent event) throws AbortProcessingException {
		logger.debug("File Upload handler routine called");
		UploadedFile file = event.getFile();
		if (file != null) { // i.e. we have success
			try {
				UploadedFileInformation fileInformation = analyzeAndCopyFile(file);
				uploadedFiles.add(fileInformation); // add to the list
				logger.info("Uploaded file: " + file.getFileName());
				FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO,
						"The files have been sucessfully uploaded. Click below to continue to import.", "");
				FacesContext.getCurrentInstance().addMessage("successInfo", facesMsg);
			} catch (IOException e) {
				FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO,
						"There was some error while uploading the files", "");
				FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			}
		} else {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, "The file " + file.getFileName()
					+ " was not sucessfully uploaded", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
		}

	}

	public void doImport() {
		FacesContext context = FacesContext.getCurrentInstance();
		HttpServletResponse response = (HttpServletResponse) context.getExternalContext().getResponse();
		response.setContentType("text/plain");
		try {
			OutputStream outStream = response.getOutputStream();
			PrintWriter out = new PrintWriter(outStream);
			DataSetImporter xmlReader = new DataSetImporter();

			out.println("------ Begin import of files ------");
			out.println("Please DO NOT reload this page until finished!");
			// 1. import all files which are plain XML-files

			long start = System.currentTimeMillis();

			xmlReader.importFiles(getUploadDirectory() + "/", out);

			for (UploadedFileInformation fileInf : uploadedFiles) {
				if (fileInf.getFileName().endsWith(".zip")) {
					if (out != null) {
						out.println("Import zip archive " + fileInf.getFileName());
						out.flush();
					}
					xmlReader.importZipFile(getUploadDirectory() + "/" + fileInf.getFileName(), out);
				}
			}

			long stop = System.currentTimeMillis();

			long duration = (stop - start) / 1000;

			logger.info("import finished in " + duration + " seconds");

			out.println("------ import of files finished ------");
			out.flush();

			// OK, clean up the resources
			deleteUploadDirectory();
			uploadedFiles.clear();

		} catch (Exception e) {
			logger.error("error: file import failed", e);
		} finally {
		}
		context.responseComplete();

	}

	public List<UploadedFileInformation> getUploadedFiles() {
		return uploadedFiles;
	}

	private void copyFile(InputStream inStream, String directory, String fileName) throws IOException {
		File dir = new File(directory);
		if (!dir.exists()) {
			FileUtils.forceMkdir(dir);
		} else {
			// cleanup the directory first
			// FileUtils.cleanDirectory(dir);
		}
		String outputFilePath;
		if (directory.endsWith("/")) {
			outputFilePath = directory + fileName;
		} else {
			outputFilePath = directory + "/" + fileName;
		}
		File outputFile = new File(outputFilePath);
		FileOutputStream outputStream = new FileOutputStream(outputFile);

		IOUtils.copy(inStream, outputStream);
		outputStream.close();
	}

	private String getUploadDirectory() {
		return ConfigurationService.INSTANCE.getUploadDirectory() + "/" + handlerId.toString();
	}

	private void deleteUploadDirectory() {

		String directory = getUploadDirectory();

		try {
			File dir = new File(directory);
			FileUtils.deleteDirectory(dir);
		} catch (IOException e) {
			logger.error("Cannot delete upload directory " + directory);
		}
	}

	private String getDataSetDirectory(ILCDTypes dataSetType) {
		String directory = getUploadDirectory();
		switch (dataSetType) {
		case CONTACT:
			directory = directory + "/" + "contacts";
			break;
		case FLOWPROPERTY:
			directory = directory + "/" + "flowproperties";
			break;
		case FLOW:
			directory = directory + "/" + "flows";
			break;
		case PROCESS:
			directory = directory + "/" + "processes";
			break;
		case SOURCE:
			directory = directory + "/" + "sources";
			break;
		case UNITGROUP:
			directory = directory + "/" + "unitgroups";
			break;
		case LCIAMETHOD:
			directory = directory + "/" + "lciamethods";
			break;
		default:
			directory = null;
		}

		return directory;
	}

	private UploadedFileInformation analyzeAndCopyFile(UploadedFile file) throws IOException {

		UploadedFileInformation fileInformation = null;

		if (file.getFileName().endsWith(".xml")) {
			fileInformation = analyzeAndCopyXmlFile(file);
		} else if (file.getFileName().endsWith(".zip")) {
			fileInformation = analyzeAndCopyArchive(file);
		} else {
			fileInformation = analyzeAndCopyOtherFile(file);
		}

		return fileInformation;

	}

	private UploadedFileInformation analyzeAndCopyXmlFile(UploadedFile file) throws IOException {
		// xml file
		UploadedFileInformation fileInformation = new UploadedFileInformation();
		fileInformation.setFileName(file.getFileName());
		fileInformation.setFileType(file.getContentType());
		fileInformation.setFileSize(file.getSize());
		InputStream in = file.getInputstream();
		DatasetDAO dataSetDao = new DatasetDAO();

		ILCDTypes type = dataSetDao.determineDatasetType(in);
		in.close();
		fileInformation.setIlcdType(type);
		if (type == null) {
			logger.error("Cannot determine file type");
			return fileInformation;
		}

		String targetDirectory = getDataSetDirectory(type);
		if (targetDirectory == null) {
			fileInformation.setMessage("Warning: The XML file does not contain an ILCD data set; file ignored");
			return fileInformation;
		}
		in = file.getInputstream();
		copyFile(in, targetDirectory, file.getFileName());
		in.close();
		fileInformation.setPathName(targetDirectory + "/" + file.getFileName());

		return fileInformation;
	}

	private UploadedFileInformation analyzeAndCopyArchive(UploadedFile file) throws IOException {
		UploadedFileInformation fileInformation = new UploadedFileInformation();
		fileInformation.setFileName(file.getFileName());
		fileInformation.setFileType(file.getContentType());
		fileInformation.setFileSize(file.getSize());

		String targetDirectory = getUploadDirectory();
		InputStream in = file.getInputstream();
		copyFile(file.getInputstream(), targetDirectory, file.getFileName());
		in.close();
		fileInformation.setPathName(targetDirectory + "/" + file.getFileName());

		return fileInformation;
	}

	private UploadedFileInformation analyzeAndCopyOtherFile(UploadedFile file) throws IOException {
		UploadedFileInformation fileInformation = new UploadedFileInformation();
		fileInformation.setFileName(file.getFileName());
		fileInformation.setFileType(file.getContentType());
		fileInformation.setFileSize(file.getSize());

		String targetDirectory = getUploadDirectory() + "/" + "external_docs";
		InputStream in = file.getInputstream();
		copyFile(file.getInputstream(), targetDirectory, file.getFileName());
		in.close();
		fileInformation.setPathName(targetDirectory + "/" + file.getFileName());

		return fileInformation;
	}
}
