/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/

package de.iai.ilcd.webgui.controller.admin;

import java.io.Serializable;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.context.FacesContext;

import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.IncorrectCredentialsException;
import org.apache.shiro.authc.LockedAccountException;
import org.apache.shiro.authc.UnknownAccountException;
import org.apache.shiro.subject.Subject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.iai.ilcd.model.dao.UserDao;
import de.iai.ilcd.security.IlcdSecurityRealm;
import de.iai.ilcd.security.UserLoginActions;

@ManagedBean
@SessionScoped
public class LoginHandler implements Serializable {

	/**
	 * Serialization ID
	 */
	private static final long serialVersionUID = -1378066230750034948L;

	/**
	 * Current input for login name
	 */
	private String loginName;

	/**
	 * Current input for password
	 */
	private String password;

	/**
	 * Current user logged in flag
	 */
	boolean loggedIn = false;

	/**
	 * DAO for user objects
	 */
	private final UserDao userDao = new UserDao();
	
	/**
	 * Logger instance
	 */
	private static Logger logger = LoggerFactory.getLogger(LoginHandler.class);

	/**
	 * Creates a new instance of LoginHandler
	 */
	public LoginHandler() {
	}

	/**
	 * Get the current login name
	 * @return current login name
	 */
	public String getLoginName() {
		Subject currentUser = SecurityUtils.getSubject();
		if (currentUser.isAuthenticated()) {
			return (String) currentUser.getPrincipal();
		} else {
			return loginName;
		}
	}

	/**
	 * Set the login name from user input
	 * @param loginName current input value
	 */
	public void setLoginName(String loginName) {
		this.loginName = loginName;
	}

	/**
	 * Get the current password
	 * @return current password
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * Set the password from user input
	 * @param password current input value
	 */
	public void setPassword(String password) {
		this.password = password;
	}

	/**
	 * Do login by user provided credentials
	 * @return always <code>null</code>
	 */
	public String login() {
		// we use the Shiro framework here

		FacesMessage message = new FacesMessage();
		message.setSeverity(FacesMessage.SEVERITY_INFO);
		FacesContext facesContext = FacesContext.getCurrentInstance();

		Subject currentUser = SecurityUtils.getSubject();
		if (currentUser.isAuthenticated()) {
			message.setSummary("You are already logged in; please logout first");
			message.setSeverity(FacesMessage.SEVERITY_WARN);
			facesContext.addMessage(null, message);
			return null;
		}

		// no user currently logged in
		if (this.loginName == null || this.password == null) {
			message.setSummary("You must supply a valid login name and password");
			message.setSeverity(FacesMessage.SEVERITY_INFO);
			facesContext.addMessage(null, message);
			return null;
		}

		String salt = this.userDao.getSalt( this.loginName );
		if ( salt == null ){
			message.setSummary("Incorrect user name and/or password; try again");
			message.setSeverity(FacesMessage.SEVERITY_ERROR);
			facesContext.addMessage(null, message);
			return null;
		}
		
		try {
			UserLoginActions loginActions = new UserLoginActions();
			loginActions.login(this.loginName, IlcdSecurityRealm.getEncryptedPassword(this.password, salt));
			facesContext.getExternalContext().redirect(
                                facesContext.getExternalContext().getRequestContextPath() + "/index.xhtml");
			/* we shouldn't do that, because also normal users (not admin's) could log in
                        <facesContext.getExternalContext().redirect(
					facesContext.getExternalContext().getRequestContextPath() + "/admin/admin.xhtml");
                         */
			// message.setSummary("Your login was successful");
		} catch (UnknownAccountException uae) {
			message.setSummary("Incorrect user name and/or password; try again");
			message.setSeverity(FacesMessage.SEVERITY_ERROR);
		} catch (IncorrectCredentialsException ice) {
			message.setSummary("Incorrect user name and/or password; try again");
			message.setSeverity(FacesMessage.SEVERITY_ERROR);
		} catch (LockedAccountException lae) {
			message.setSummary("Your account is currently locked. Contact an administrator.");
			message.setSeverity(FacesMessage.SEVERITY_ERROR);
		} catch (AuthenticationException au) {
			logger.info(au.getMessage());
			message.setSummary("Incorrect user name and/or password; try again");
			message.setSeverity(FacesMessage.SEVERITY_ERROR);
		} catch (Exception e) {
			message.setSummary("An error occurred; Please try again");
			message.setSeverity(FacesMessage.SEVERITY_ERROR);
			message.setDetail(e.getMessage());
		}

		facesContext.addMessage(null, message);
		return null;
	}

	/**
	 * Logout the current user
	 */
	public void logout() {
		FacesMessage message = new FacesMessage();
		message.setSeverity(FacesMessage.SEVERITY_INFO);
		FacesContext facesContext = FacesContext.getCurrentInstance();

		UserLoginActions loginActions = new UserLoginActions();
		loginActions.logout();

		message.setSummary("you are now logged out");
		facesContext.addMessage("null", message);
		return;

	}

	/**
	 * Determine if a user is currently logged in. Delegates to {@link UserLoginActions#isLoggedIn()}.
	 * @return <code>true</code> if user is logged in, else <code>false</code>
	 */
	public boolean isLoggedIn() {
		UserLoginActions loginActions = new UserLoginActions();

		return loginActions.isLoggedIn();
	}
}
