/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.webgui.controller.admin;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import org.apache.velocity.tools.generic.ValueParser;
import org.primefaces.component.datatable.DataTable;
import org.primefaces.model.LazyDataModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.iai.ilcd.model.common.DataSetState;
import de.iai.ilcd.model.dao.DeleteDataSetException;
import de.iai.ilcd.model.dao.MergeException;
import de.iai.ilcd.model.dao.ProcessDao;
import de.iai.ilcd.model.process.Process;

/**
 * 
 * @author clemens.duepmeier
 */
@ManagedBean()
@ViewScoped
public class ProcessListHandler implements Serializable {

	private static Logger logger = LoggerFactory.getLogger(ProcessListHandler.class);

	private LazyDataModel<Process> lazyModel;

	private Process[] selectedProcesses;

	ValueParser params;

	public ProcessListHandler() {
		lazyModel = new ProcessDataModel();

		ProcessDao dao = new ProcessDao();
		params = new ValueParser();
		long rowCount = dao.searchResultCount(params, false);
		lazyModel.setRowCount((int) rowCount);
	}

	public LazyDataModel<Process> getLazyModel() {
		return lazyModel;
	}

	public void setLazyModel(LazyDataModel<Process> lazyModel) {
		this.lazyModel = lazyModel;
	}

	public Process[] getSelectedProcesses() {
		return selectedProcesses;
	}

	public void setSelectedProcesses(Process[] selectedProcesses) {
		this.selectedProcesses = selectedProcesses;
	}

	public void deleteSelected() {
		if (selectedProcesses == null) {
			return;
		}

		ProcessDao pDao = new ProcessDao();

		for (Process process : selectedProcesses) {
			try {
				pDao.remove(process);
			} catch (DeleteDataSetException ex) {
				FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR, "Could not remove process "
						+ process.getProcessName().getName() + "\n", "");
				FacesContext.getCurrentInstance().addMessage("error", facesMsg);
			}
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, "Removed process "
					+ process.getProcessName().getName() + "\n", "");
			FacesContext.getCurrentInstance().addMessage("info", facesMsg);
		}

		selectedProcesses = null;

		ProcessDao dao = new ProcessDao();
		int rowCount = (int) dao.searchResultCount(params, false);
		lazyModel.setRowCount(rowCount);
		DataTable dataTable = (DataTable) FacesContext.getCurrentInstance().getViewRoot()
				.findComponent("tableForm:processTable");
		if (dataTable != null) {
			dataTable.loadLazyData();
		}

		return;
	}

	public void releaseSelected() {
		changeReleaseState(DataSetState.RELEASED, false);
	}

	public void unreleaseSelected() {
		changeReleaseState(DataSetState.UNRELEASED, false);
	}

	public void unlockSelected() {
		changeReleaseState(DataSetState.UNRELEASED, true);
	}

	private void changeReleaseState(DataSetState state, boolean forceUnlock) {
		if (selectedProcesses == null) {
			return;
		}

		ProcessDao pDao = new ProcessDao();

		for (Process process : selectedProcesses) {
			try {
				if (process.getReleaseState().equals(state))
					continue;
				if (process.getReleaseState().equals(DataSetState.LOCKED) && !forceUnlock)
					continue; // Locked data sets will only change state they should be explicitely unlocked
				process.setReleaseState(state);
				pDao.merge(process);
			} catch (MergeException e) {
				FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR,
						"Unknown error: Could not save release state of " + process.getProcessName().getName()
								+ " into database\n", "");
				FacesContext.getCurrentInstance().addMessage("error", facesMsg);
			}
		}
		FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, "Changed state of data sets", "");
		FacesContext.getCurrentInstance().addMessage("info", facesMsg);

		selectedProcesses = null;

		ProcessDao dao = new ProcessDao();
		int rowCount = (int) dao.searchResultCount(params,false);
		lazyModel.setRowCount(rowCount);
		DataTable dataTable = (DataTable) FacesContext.getCurrentInstance().getViewRoot()
				.findComponent("tableForm:processTable");
		// int page = dataTable.getPage();
		if (dataTable != null) {
			dataTable.loadLazyData();
		}

		return;

	}

	private class ProcessDataModel extends LazyDataModel<Process> {
		// we have to overload the load()-method of the lazy data model

		@Override
		public List<Process> load(int first, int pageSize, String sortField, boolean sortOrder,
				Map<String, String> filters) {
			logger.info("Loading the lazy process data between {0} and {1}", new Object[] { first, (first + pageSize) });

			// Sorting and Filtering information are not used for demo purposes just random dummy data is returned

			List<Process> lazyProcesses = new ArrayList<Process>();
			// now let's fill it from the database
			ProcessDao dao = new ProcessDao();
			// ValueParser params = new ValueParser();
			lazyProcesses = dao.search(Process.class, params, first, pageSize, sortField,false);

			// long rowCount = dao.searchResultCount(params);
			// this.setRowCount((int) rowCount);
			// logger.error("row count for process table: {}", rowCount);
			return lazyProcesses;
		}
	}
}
